/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.jpdl.patterns;

import junit.framework.TestCase;

import org.jbpm.graph.def.ActionHandler;
import org.jbpm.graph.def.Node;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ExecutionContext;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;
import org.jbpm.taskmgmt.def.Task;
import org.jbpm.taskmgmt.def.TaskMgmtDefinition;
import org.jbpm.taskmgmt.exe.TaskInstance;
import org.jbpm.taskmgmt.exe.TaskMgmtInstance;

/**
 * http://is.tm.tue.nl/research/patterns/download/swf/pat_15.swf
 */
public class Wfp16MiWithoutAPrioriRuntimeKnowledge extends TestCase {
  
  public static int scenario = -1;
  
  public static class CreateTasks implements ActionHandler {
    private static final long serialVersionUID = 1L;
    public void execute(ExecutionContext executionContext) throws Exception {
      TaskMgmtDefinition tmd = (TaskMgmtDefinition) executionContext.getDefinition(TaskMgmtDefinition.class);
      Task task = tmd.getTask("watch movie amadeus");
    
      // create as many task instances as the scenario prescribes : 
      //  0 tasks for scenario 1
      //  1 task for scenario 2
      //  2 tasks for scenario 3
      //  3 tasks for scenario 4
      TaskMgmtInstance tmi = executionContext.getTaskMgmtInstance();
      for (int i = 1; i<scenario; i++) {
        tmi.createTaskInstance(task, executionContext.getToken());
      }
    }
  }

  public static ProcessDefinition processDefinition = ProcessDefinition.parseXmlString(
    "<process-definition>" +
    "  <start-state name='a'>" +
    "    <transition to='b' />" +
    "  </start-state>" +
    "  <state name='b'>" +
    "    <transition to='t' />" +
    "  </state>" +
    "  <task-node name='t' create-tasks='false' signal='last-wait'>" +
    "    <event type='node-enter'>" +
    "      <action class='org.jbpm.jpdl.patterns.Wfp16MiWithoutAPrioriRuntimeKnowledge$CreateTasks' />" +
    "    </event>" +
    "    <task name='watch movie amadeus' />" +
    "    <transition to='c' />" +
    "  </task-node>" +
    "  <state name='c' />" +
    "</process-definition>"
  );
  
  public static Node t = processDefinition.getNode("t");
  public static Node c = processDefinition.getNode("c");

  public void testAprioriRuntimeKnowledgeScenario1() {
    scenario = 1;
    
    ProcessInstance processInstance = new ProcessInstance(processDefinition);
    Token token = processInstance.getRootToken();
    processInstance.signal();
    processInstance.signal();
    
    assertSame(t, token.getNode());
    assertEquals(0, getNbrOfTasks(token));
    
    addOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    addOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(2, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(c, token.getNode());
    assertEquals(0, getNbrOfTasks(token));
  }

  public void testAprioriRuntimeKnowledgeScenario2() {
    scenario = 2;
    
    ProcessInstance processInstance = new ProcessInstance(processDefinition);
    Token token = processInstance.getRootToken();
    processInstance.signal();
    processInstance.signal();
    
    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    addOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(2, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(c, token.getNode());
    assertEquals(0, getNbrOfTasks(token));
  }

  public void testAprioriRuntimeKnowledgeScenario3() {
    scenario = 3;
    
    ProcessInstance processInstance = new ProcessInstance(processDefinition);
    Token token = processInstance.getRootToken();
    processInstance.signal();
    processInstance.signal();
    
    assertSame(t, token.getNode());
    assertEquals(2, getNbrOfTasks(token));
    
    addOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(3, getNbrOfTasks(token));
    
    endOneTask(token);
    endOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(c, token.getNode());
    assertEquals(0, getNbrOfTasks(token));
  }

  public void testAprioriRuntimeKnowledgeScenario4() {
    scenario = 4;
    
    ProcessInstance processInstance = new ProcessInstance(processDefinition);
    Token token = processInstance.getRootToken();
    processInstance.signal();
    processInstance.signal();
    
    assertSame(t, token.getNode());
    assertEquals(3, getNbrOfTasks(token));
    
    addOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(4, getNbrOfTasks(token));
    
    endOneTask(token);
    endOneTask(token);
    endOneTask(token);

    assertSame(t, token.getNode());
    assertEquals(1, getNbrOfTasks(token));
    
    endOneTask(token);

    assertSame(c, token.getNode());
    assertEquals(0, getNbrOfTasks(token));
  }

  private void addOneTask(Token token) {
    TaskMgmtDefinition tmd = (TaskMgmtDefinition) processDefinition.getDefinition(TaskMgmtDefinition.class);
    Task task = tmd.getTask("watch movie amadeus");
    
    TaskMgmtInstance tmi = token.getProcessInstance().getTaskMgmtInstance();
    tmi.createTaskInstance(task, token);
  }

  private int getNbrOfTasks(Token token) {
    TaskMgmtInstance tmi = (TaskMgmtInstance) token.getProcessInstance().getInstance(TaskMgmtInstance.class);
    return tmi.getUnfinishedTasks(token).size();
  }

  public static void endOneTask(Token token) {
    TaskMgmtInstance tmi = (TaskMgmtInstance)token.getProcessInstance().getInstance(TaskMgmtInstance.class);
    TaskInstance taskInstance = (TaskInstance) tmi.getUnfinishedTasks(token).iterator().next();
    taskInstance.end();
  }
}
