/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.webapp.bean;

import java.util.List;

import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.db.TaskMgmtSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.taskmgmt.exe.TaskInstance;

public class HomeBean {

  UserBean userBean;
  TaskBean taskBean;
  
  JbpmContext jbpmContext;
  GraphSession graphSession;
  TaskMgmtSession taskMgmtSession;
  
  public HomeBean() {
    this.jbpmContext = JbpmContext.getCurrentJbpmContext();
    this.graphSession = jbpmContext.getGraphSession();
    this.taskMgmtSession = jbpmContext.getTaskMgmtSession();
  }
  
  public List getTaskInstances() {
    return taskMgmtSession.findTaskInstances(userBean.getUserName());
  }

  public List getLatestProcessDefinitions() {
    return graphSession.findLatestProcessDefinitions();
  }

  /**
   * selects a task.
   */
  public String selectTaskInstance() {
    // Get the task instance id from request parameter
    long taskInstanceId = JsfHelper.getId("taskInstanceId");
    TaskInstance taskInstance = taskMgmtSession.loadTaskInstance(taskInstanceId);
    taskBean.initialize(taskInstance);
    return "task";
  }

  /**
   * prepares a task form for starting a new process instance.
   */
  public String startProcessInstance() {
    // Get the task instance id from request parameter
    long processDefinitionId = JsfHelper.getId("processDefinitionId");
    ProcessDefinition processDefinition = graphSession.loadProcessDefinition(processDefinitionId);

    // create a new process instance to run
    ProcessInstance processInstance = new ProcessInstance(processDefinition);

    // create a new taskinstance for the start task
    TaskInstance taskInstance = processInstance.getTaskMgmtInstance().createStartTaskInstance();

    // Save the process instance along with the task instance
    jbpmContext.save(processInstance);
    
    // Fill the task backing bean with useful information
    taskBean.initialize(taskInstance);

    return "task";
  }
  
  public UserBean getUserBean() {
    return userBean;
  }
  public void setUserBean(UserBean userBean) {
    this.userBean = userBean;
  }
  public TaskBean getTaskBean() {
    return taskBean;
  }
  public void setTaskBean(TaskBean taskBean) {
    this.taskBean = taskBean;
  }
}
